@@ -1534,8 +1534,20 @@ static pa_bool_t sink_set_formats(pa_sink *s, pa_idxset *formats) {
     pa_idxset_free(u->formats, (pa_free2_cb_t) pa_format_info_free2, NULL);
     u->formats = pa_idxset_new(NULL, NULL);
 
+    /* Note: the logic below won't apply if we're using software encoding.
+     * This is fine for now since we don't support that via the passthrough
+     * framework, but this must be changed if we do. */
+
+    /* First insert non-PCM formats since we prefer those. */
     PA_IDXSET_FOREACH(f, formats, idx) {
-        pa_idxset_put(u->formats, pa_format_info_copy(f), NULL);
+        if (!pa_format_info_is_pcm(f))
+            pa_idxset_put(u->formats, pa_format_info_copy(f), NULL);
+    }
+
+    /* Now add any PCM formats */
+    PA_IDXSET_FOREACH(f, formats, idx) {
+        if (pa_format_info_is_pcm(f))
+            pa_idxset_put(u->formats, pa_format_info_copy(f), NULL);
     }
 
     return TRUE;
@@ -1698,8 +1710,12 @@ static void thread_func(void *userdata) {
         if (u->sink->flags & PA_SINK_DEFERRED_VOLUME) {
             pa_usec_t volume_sleep;
             pa_sink_volume_change_apply(u->sink, &volume_sleep);
-            if (volume_sleep > 0)
-                rtpoll_sleep = PA_MIN(volume_sleep, rtpoll_sleep);
+            if (volume_sleep > 0) {
+                if (rtpoll_sleep > 0)
+                    rtpoll_sleep = PA_MIN(volume_sleep, rtpoll_sleep);
+                else
+                    rtpoll_sleep = volume_sleep;
+            }
         }
 
         if (rtpoll_sleep > 0)
