@@ -25,14 +25,12 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 
 #include <pulse/xmalloc.h>
 #include <pulsecore/module.h>
 #include <pulsecore/core-util.h>
 #include <pulsecore/modargs.h>
 #include <pulsecore/macro.h>
-#include <pulsecore/llist.h>
 #include <pulsecore/core-util.h>
 #include <pulsecore/dbus-shared.h>
 
@@ -42,21 +40,14 @@
 PA_MODULE_AUTHOR("Joao Paulo Rechi Vita");
 PA_MODULE_DESCRIPTION("Detect available bluetooth audio devices and load bluetooth audio drivers");
 PA_MODULE_VERSION(PACKAGE_VERSION);
-PA_MODULE_USAGE("async=<Asynchronous initialization?>");
+PA_MODULE_USAGE("async=<Asynchronous initialization?> "
+                "sco_sink=<name of sink> "
+                "sco_source=<name of source> ");
 PA_MODULE_LOAD_ONCE(TRUE);
 
-/*
-#ifdef NOKIA
-   "sco_sink=<name of sink> "
-   "sco_source=<name of source>"
-#endif
-*/
-
 static const char* const valid_modargs[] = {
-#ifdef NOKIA
     "sco_sink",
     "sco_source",
-#endif
     "async",
     NULL
 };
@@ -83,8 +74,10 @@ static pa_hook_result_t load_module_for_device(pa_bluetooth_discovery *y, const
 
     mi = pa_hashmap_get(u->hashmap, d->path);
 
-    if (!d->dead &&
-        d->device_connected > 0 && (d->audio_state >= PA_BT_AUDIO_STATE_CONNECTED || d->audio_source_state >= PA_BT_AUDIO_STATE_CONNECTED)) {
+    if (!d->dead && d->device_connected > 0 &&
+        (d->audio_state >= PA_BT_AUDIO_STATE_CONNECTED ||
+         d->audio_source_state >= PA_BT_AUDIO_STATE_CONNECTED ||
+         d->hfgw_state > PA_BT_AUDIO_STATE_CONNECTED)) {
 
         if (!mi) {
             pa_module *m = NULL;
@@ -103,7 +96,6 @@ static pa_hook_result_t load_module_for_device(pa_bluetooth_discovery *y, const
             }
 #endif
 
-#ifdef NOKIA
             if (pa_modargs_get_value(u->modargs, "sco_sink", NULL) &&
                 pa_modargs_get_value(u->modargs, "sco_source", NULL)) {
                 char *tmp;
@@ -114,10 +106,12 @@ static pa_hook_result_t load_module_for_device(pa_bluetooth_discovery *y, const
                 pa_xfree(args);
                 args = tmp;
             }
-#endif
 
             if (d->audio_source_state >= PA_BT_AUDIO_STATE_CONNECTED)
-                args = pa_sprintf_malloc("%s profile=\"a2dp_source\"", args);
+                args = pa_sprintf_malloc("%s profile=\"a2dp_source\" auto_connect=no", args);
+
+            if (d->hfgw_state > PA_BT_AUDIO_STATE_CONNECTED)
+                args = pa_sprintf_malloc("%s profile=\"hfgw\"", args);
 
             pa_log_debug("Loading module-bluetooth-device %s", args);
             m = pa_module_load(u->module->core, "module-bluetooth-device", args);
