@@ -27,10 +27,15 @@
 #include <string.h>
 #include <errno.h>
 #include <sys/types.h>
-#include <regex.h>
 #include <stdio.h>
 #include <stdlib.h>
 
+#if defined(HAVE_REGEX_H)
+#include <regex.h>
+#elif defined(HAVE_PCREPOSIX_H)
+#include <pcreposix.h>
+#endif
+
 #include <pulse/xmalloc.h>
 
 #include <pulsecore/core-error.h>
@@ -38,7 +43,6 @@
 #include <pulsecore/core-util.h>
 #include <pulsecore/modargs.h>
 #include <pulsecore/log.h>
-#include <pulsecore/core-subscribe.h>
 #include <pulsecore/sink-input.h>
 #include <pulsecore/core-util.h>
 
@@ -56,6 +60,9 @@ PA_MODULE_USAGE("table=<filename> "
 #define DEFAULT_MATCH_TABLE_FILE PA_DEFAULT_CONFIG_DIR"/match.table"
 #define DEFAULT_MATCH_TABLE_FILE_USER "match.table"
 
+#define UPDATE_REPLACE "replace"
+#define UPDATE_MERGE "merge"
+
 static const char* const valid_modargs[] = {
     "table",
     "key",
@@ -66,13 +73,14 @@ struct rule {
     regex_t regex;
     pa_volume_t volume;
     pa_proplist *proplist;
+    pa_update_mode_t mode;
     struct rule *next;
 };
 
 struct userdata {
     struct rule *rules;
     char *property_key;
-    pa_subscription *subscription;
+    pa_hook_slot *sink_input_new_hook_slot;
 };
 
 static int load_rules(struct userdata *u, const char *filename) {
@@ -85,12 +93,11 @@ static int load_rules(struct userdata *u, const char *filename) {
     pa_assert(u);
 
     if (filename)
-        f = fopen(fn = pa_xstrdup(filename), "r");
+        f = pa_fopen_cloexec(fn = pa_xstrdup(filename), "r");
     else
         f = pa_open_config_file(DEFAULT_MATCH_TABLE_FILE, DEFAULT_MATCH_TABLE_FILE_USER, NULL, &fn);
 
     if (!f) {
-        pa_xfree(fn);
         pa_log("Failed to open file config file: %s", pa_cstrerror(errno));
         goto finish;
     }
@@ -98,13 +105,14 @@ static int load_rules(struct userdata *u, const char *filename) {
     pa_lock_fd(fileno(f), 1);
 
     while (!feof(f)) {
-        char *d, *v;
+        char *token_end, *value_str;
         pa_volume_t volume = PA_VOLUME_NORM;
         uint32_t k;
         regex_t regex;
         char ln[256];
         struct rule *rule;
         pa_proplist *proplist = NULL;
+        pa_update_mode_t mode = (pa_update_mode_t) -1;
 
         if (!fgets(ln, sizeof(ln), f))
             break;
@@ -116,51 +124,72 @@ static int load_rules(struct userdata *u, const char *filename) {
         if (ln[0] == '#' || !*ln )
             continue;
 
-        d = ln+strcspn(ln, WHITESPACE);
-        v = d+strspn(d, WHITESPACE);
+        token_end = ln + strcspn(ln, WHITESPACE);
+        value_str = token_end + strspn(token_end, WHITESPACE);
+        *token_end = 0;
 
-
-        if (!*v) {
-            pa_log(__FILE__ ": [%s:%u] failed to parse line - too few words", filename, n);
+        if (!*ln) {
+            pa_log("[%s:%u] failed to parse line - missing regexp", fn, n);
             goto finish;
         }
 
-        *d = 0;
-        if (pa_atou(v, &k) >= 0) {
-            volume = (pa_volume_t) k;
-        } else if (*v == '"') {
-            char *e;
+        if (!*value_str) {
+            pa_log("[%s:%u] failed to parse line - too few words", fn, n);
+            goto finish;
+        }
 
-            e = strchr(v+1, '"');
-            if (!e) {
-                pa_log(__FILE__ ": [%s:%u] failed to parse line - missing role closing quote", filename, n);
-                goto finish;
-            }
+        if (pa_atou(value_str, &k) >= 0)
+            volume = (pa_volume_t) PA_CLAMP_VOLUME(k);
+        else {
+            size_t len;
 
-            *e = '\0';
-            e = pa_sprintf_malloc("media.role=\"%s\"", v+1);
-            proplist = pa_proplist_from_string(e);
-            pa_xfree(e);
-        } else {
-            char *e;
+            token_end = value_str + strcspn(value_str, WHITESPACE);
 
-            e = v+strspn(v, WHITESPACE);
-            if (!*e) {
-                pa_log(__FILE__ ": [%s:%u] failed to parse line - missing end of property list", filename, n);
-                goto finish;
-            }
-            *e = '\0';
-            proplist = pa_proplist_from_string(v);
+            len = token_end - value_str;
+            if (len == (sizeof(UPDATE_REPLACE) - 1) && !strncmp(value_str, UPDATE_REPLACE, len))
+                mode = PA_UPDATE_REPLACE;
+            else if (len == (sizeof(UPDATE_MERGE) - 1) && !strncmp(value_str, UPDATE_MERGE, len))
+                mode = PA_UPDATE_MERGE;
+
+            if (mode != (pa_update_mode_t) -1) {
+                value_str = token_end + strspn(token_end, WHITESPACE);
+
+                if (!*value_str) {
+                    pa_log("[%s:%u] failed to parse line - too few words", fn, n);
+                    goto finish;
+                }
+            } else
+                mode = PA_UPDATE_MERGE;
+
+            if (*value_str == '"') {
+                value_str++;
+
+                token_end = strchr(value_str, '"');
+                if (!token_end) {
+                    pa_log("[%s:%u] failed to parse line - missing role closing quote", fn, n);
+                    goto finish;
+                }
+            } else
+                token_end = value_str + strcspn(value_str, WHITESPACE);
+
+            *token_end = 0;
+
+            value_str = pa_sprintf_malloc("media.role=\"%s\"", value_str);
+            proplist = pa_proplist_from_string(value_str);
+            pa_xfree(value_str);
         }
 
         if (regcomp(&regex, ln, REG_EXTENDED|REG_NOSUB) != 0) {
-            pa_log("[%s:%u] invalid regular expression", filename, n);
+            pa_log("[%s:%u] invalid regular expression", fn, n);
+            if (proplist)
+                pa_proplist_free(proplist);
             goto finish;
         }
 
         rule = pa_xnew(struct rule, 1);
         rule->regex = regex;
         rule->proplist = proplist;
+        rule->mode = mode;
         rule->volume = volume;
         rule->next = NULL;
 
@@ -169,8 +198,6 @@ static int load_rules(struct userdata *u, const char *filename) {
         else
             u->rules = rule;
         end = rule;
-
-        *d = 0;
     }
 
     ret = 0;
@@ -181,29 +208,20 @@ finish:
         fclose(f);
     }
 
-    if (fn)
-        pa_xfree(fn);
+    pa_xfree(fn);
 
     return ret;
 }
 
-static void callback(pa_core *c, pa_subscription_event_type_t t, uint32_t idx, void *userdata) {
-    struct userdata *u =  userdata;
-    pa_sink_input *si;
+static pa_hook_result_t sink_input_new_hook_callback(pa_core *c, pa_sink_input_new_data *si, struct userdata *u) {
     struct rule *r;
     const char *n;
 
     pa_assert(c);
     pa_assert(u);
 
-    if (t != (PA_SUBSCRIPTION_EVENT_SINK_INPUT|PA_SUBSCRIPTION_EVENT_NEW))
-        return;
-
-    if (!(si = pa_idxset_get_by_index(c->sink_inputs, idx)))
-        return;
-
     if (!(n = pa_proplist_gets(si->proplist, u->property_key)))
-        return;
+        return PA_HOOK_OK;
 
     pa_log_debug("Matching with %s", n);
 
@@ -211,15 +229,18 @@ static void callback(pa_core *c, pa_subscription_event_type_t t, uint32_t idx, v
         if (!regexec(&r->regex, n, 0, NULL, 0)) {
             if (r->proplist) {
                 pa_log_debug("updating proplist of sink input '%s'", n);
-                pa_proplist_update(si->proplist, PA_UPDATE_MERGE, r->proplist);
-            } else {
+                pa_proplist_update(si->proplist, r->mode, r->proplist);
+            } else if (si->volume_writable) {
                 pa_cvolume cv;
                 pa_log_debug("changing volume of sink input '%s' to 0x%03x", n, r->volume);
                 pa_cvolume_set(&cv, si->sample_spec.channels, r->volume);
-                pa_sink_input_set_volume(si, &cv, TRUE, FALSE);
-            }
+                pa_sink_input_new_data_set_volume(si, &cv);
+            } else
+                pa_log_debug("the volume of sink input '%s' is not writable, can't change it", n);
         }
     }
+
+    return PA_HOOK_OK;
 }
 
 int pa__init(pa_module*m) {
@@ -233,9 +254,8 @@ int pa__init(pa_module*m) {
         goto fail;
     }
 
-    u = pa_xnew(struct userdata, 1);
+    u = pa_xnew0(struct userdata, 1);
     u->rules = NULL;
-    u->subscription = NULL;
     m->userdata = u;
 
     u->property_key = pa_xstrdup(pa_modargs_get_value(ma, "key", PA_PROP_MEDIA_NAME));
@@ -243,10 +263,8 @@ int pa__init(pa_module*m) {
     if (load_rules(u, pa_modargs_get_value(ma, "table", NULL)) < 0)
         goto fail;
 
-    /* FIXME: Doing this asynchronously is just broken. This needs to
-     * use a hook! */
-
-    u->subscription = pa_subscription_new(m->core, PA_SUBSCRIPTION_MASK_SINK_INPUT, callback, u);
+    /* hook EARLY - 1, to match before stream-restore */
+    u->sink_input_new_hook_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_NEW], PA_HOOK_EARLY - 1, (pa_hook_cb_t) sink_input_new_hook_callback, u);
 
     pa_modargs_free(ma);
     return 0;
@@ -256,7 +274,7 @@ fail:
 
     if (ma)
         pa_modargs_free(ma);
-    return  -1;
+    return -1;
 }
 
 void pa__done(pa_module*m) {
@@ -268,8 +286,8 @@ void pa__done(pa_module*m) {
     if (!(u = m->userdata))
         return;
 
-    if (u->subscription)
-        pa_subscription_free(u->subscription);
+    if (u->sink_input_new_hook_slot)
+        pa_hook_slot_free(u->sink_input_new_hook_slot);
 
     if (u->property_key)
         pa_xfree(u->property_key);
