@@ -27,11 +27,11 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <string.h>
 
 #include <windows.h>
+#include <winsock2.h>
 
-extern pa_set_root(HANDLE handle);
+extern char *pa_win32_get_toplevel(HANDLE handle);
 
 BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {
     WSADATA data;
@@ -39,7 +39,7 @@ BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {
     switch (fdwReason) {
 
     case DLL_PROCESS_ATTACH:
-        if (!pa_set_root(hinstDLL))
+        if (!pa_win32_get_toplevel(hinstDLL))
             return FALSE;
         WSAStartup(MAKEWORD(2, 0), &data);
         break;
