@@ -9,7 +9,7 @@
 
   PulseAudio is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published
-  by the Free Software Foundation; either version 2 of the License,
+  by the Free Software Foundation; either version 2.1 of the License,
   or (at your option) any later version.
 
   PulseAudio is distributed in the hope that it will be useful, but
@@ -23,7 +23,6 @@
   USA.
 ***/
 
-#include <string.h>
 #include <sys/types.h>
 
 #include <pulse/xmalloc.h>
@@ -34,21 +33,23 @@ typedef struct pa_object pa_object;
 
 struct pa_object {
     PA_REFCNT_DECLARE;
-    const char *type_name;
+    const char *type_id;
     void (*free)(pa_object *o);
-    int (*check_type)(const char *type_name);
+    pa_bool_t (*check_type)(const char *type_name);
 };
 
-pa_object *pa_object_new_internal(size_t size, const char *type_name, int (*check_type)(const char *type_name));
-#define pa_object_new(type) ((type*) pa_object_new_internal(sizeof(type), #type, type##_check_type)
+pa_object *pa_object_new_internal(size_t size, const char *type_id, pa_bool_t (*check_type)(const char *type_id));
+#define pa_object_new(type) ((type*) pa_object_new_internal(sizeof(type), type##_type_id, type##_check_type)
 
 #define pa_object_free ((void (*) (pa_object* _obj)) pa_xfree)
 
-int pa_object_check_type(const char *type);
+pa_bool_t pa_object_check_type(const char *type_id);
 
-static inline int pa_object_isinstance(void *o) {
+extern const char pa_object_type_id[];
+
+static inline pa_bool_t pa_object_isinstance(void *o) {
     pa_object *obj = (pa_object*) o;
-    return obj ? obj->check_type("pa_object") : 0;
+    return obj ? obj->check_type(pa_object_type_id) : TRUE;
 }
 
 pa_object *pa_object_ref(pa_object *o);
@@ -60,7 +61,7 @@ static inline int pa_object_refcnt(pa_object *o) {
 
 static inline pa_object* pa_object_cast(void *o) {
     pa_object *obj = (pa_object*) o;
-    pa_assert(!obj || obj->check_type("pa_object"));
+    pa_assert(!obj || obj->check_type(pa_object_type_id));
     return obj;
 }
 
@@ -68,10 +69,10 @@ static inline pa_object* pa_object_cast(void *o) {
 
 #define PA_OBJECT(o) pa_object_cast(o)
 
-#define PA_DECLARE_CLASS(c)                                             \
-    static inline int c##_isinstance(void *o) {                         \
+#define PA_DECLARE_CLASS_COMMON(c)                                      \
+    static inline pa_bool_t c##_isinstance(void *o) {                   \
         pa_object *obj = (pa_object*) o;                                \
-        return obj ? obj->check_type(#c) : 1;                           \
+        return obj ? obj->check_type(c##_type_id) : TRUE;               \
     }                                                                   \
     static inline c* c##_cast(void *o) {                                \
         pa_assert(c##_isinstance(o));                                   \
@@ -91,12 +92,27 @@ static inline pa_object* pa_object_cast(void *o) {
     }                                                                   \
     struct __stupid_useless_struct_to_allow_trailing_semicolon
 
-#define PA_DEFINE_CHECK_TYPE(c, parent)                                 \
-    int c##_check_type(const char *type) {                              \
-        pa_assert(type);                                                \
-        if (strcmp(type, #c) == 0)                                      \
-            return 1;                                                   \
-        return parent##_check_type(type);                               \
+#define PA_DECLARE_PUBLIC_CLASS(c)                                      \
+    extern const char c##_type_id[];                                    \
+    PA_DECLARE_CLASS_COMMON(c);                                         \
+    pa_bool_t c##_check_type(const char *type_id)
+
+#define PA_DEFINE_PUBLIC_CLASS(c, parent)                               \
+    const char c##_type_id[] = #c;                                      \
+    pa_bool_t c##_check_type(const char *type_id) {                     \
+        if (type_id == c##_type_id)                                     \
+            return TRUE;                                                \
+        return parent##_check_type(type_id);                            \
+    }                                                                   \
+    struct __stupid_useless_struct_to_allow_trailing_semicolon
+
+#define PA_DEFINE_PRIVATE_CLASS(c, parent)                              \
+    static const char c##_type_id[] = #c;                               \
+    PA_DECLARE_CLASS_COMMON(c);                                         \
+    static pa_bool_t c##_check_type(const char *type_id) {              \
+        if (type_id == c##_type_id)                                     \
+            return TRUE;                                                \
+        return parent##_check_type(type_id);                            \
     }                                                                   \
     struct __stupid_useless_struct_to_allow_trailing_semicolon
 
