@@ -1,10 +1,10 @@
 #include <stdio.h>
 
 #include <pulse/xmalloc.h>
-#include <pulsecore/strlist.h>
-#include <pulsecore/gccmacro.h>
 
-int main(PA_GCC_UNUSED int argc, PA_GCC_UNUSED char* argv[]) {
+#include <pulsecore/strlist.h>
+
+int main(int argc, char* argv[]) {
     char *t, *u;
     pa_strlist *l = NULL;
 
